#include <cerrno>
#include <cstdio>
#include <cstdlib>
#include <sys/stat.h>

#include <Copter/Cosmology.h>
#include <Copter/LinearPS.h>


int main(int argc, char* argv[]) {
    /* Set parameters */
    const char* cstr = "example";
    real z = 0;
    int Nk = 1000;
    real kmin = 1e-4;
    real kmax = 10;
    const char* output = "linear.dat";

    /* Open output file */
    FILE* fp = fopen(output, "w");
    fprintf(fp, "# Linear theory for %s cosmology at z = %g\n", cstr, z);
    fprintf(fp, "# k       -- P_L\n");

    /* Calculate SPT power spectrum */
    Cosmology C(cstr);
    LinearPS P_L(C, z);
    for(int i = 0; i < Nk; i++) {
        real k = kmin * exp(i*log(kmax/kmin)/(Nk-1));
        fprintf(fp, "%e %e\n", k, P_L(k));
    }

    fclose(fp);
    return 0;
}
