#ifndef COMMON_H
#define COMMON_H

#ifndef NULL
#define NULL 0
#endif

/* Type definitions */
typedef double real;
typedef unsigned char uchar;
typedef unsigned short ushort;
typedef unsigned int uint;
typedef unsigned long ulong;

/* Forward declarations */
class CorrelationFunction;
class Cosmology;
class PowerSpectrum;
class Spline;
class array;
class pstring;

/* Units */
namespace SIUnits {
    const real second = 1;
    const real meter = 1;
    const real km = 1000 * meter;
    const real Mpc = 3.08568025e19 * km;

    const real c = 299792458;           // speed of light
    const real G = 6.67428e-11;         // Newton's constant
    const real hbar = 1.054571628e-34;  // Planck's constant (over 2*pi)
    const real k = 1.3806504e-23;       // Boltzmann constant

    const real t_100 = 3.08568e17;      // 1/(100 km/sec/Mpc) in seconds
}

/* Convenient print functions (that flush the buffer afterwards) */
/*  print to stdout */
void info(const char* format, ...);
/*  print to stdout if VERBOSE is defined */
void verbose(const char* format, ...);
/*  print to stdout if DEBUG is defined */
void debug(const char* format, ...);
/*  print to stderr */
void warning(const char* format, ...);
/*  print to stderr and abort */
void error(const char* format, ...);
#endif // COMMON_H
