#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <cassert>
#include <cstdio>

#include "Datafile.h"
#include "pstring.h"


Datafile::Datafile() {
    fname = "";
}

Datafile::Datafile(FILE* fp) {
    fname = "";
    Open(fp);
}

Datafile::Datafile(const char* filename) {
    Open(filename);
}

Datafile::~Datafile() {
}

bool Datafile::Open(const char* filename) {
    fname = filename;
    FILE* fp = fopen(fname, "r");
    Open(fp);
}

bool Datafile::Open(FILE* fp) {
    header = footer = "";
    columns.clear();
    ncol = nrow = 0;

    if(!fp) {
        fprintf(stderr, "Datafile: could not open file %s\n", fname);
        return false;
    }

    char buf[1024];
    vector<pstring> fields;
    while(fgets(buf, sizeof(buf), fp) != NULL) {
        pstring line = buf;
        if(line.empty() || line.startswith('#')) {
            /* Add comment lines to header or footer, depending on whether or
             * not we've reached the first line of data */
            if(nrow == 0)
                header += line;
            else
                footer += line;
            continue;
        }

        /* Split line into fields */
        fields = line.split();
        if(ncol == 0) {
            ncol = (int)fields.size();
            columns.resize(ncol);
        }
        if((int)fields.size() != ncol) {
            fprintf(stderr, "Datafile: expecting %d fields\n  line = '%s'", ncol, line.c_str());
            continue;
        }

        /* Add fields to column vectors */
        for(int j = 0; j < ncol; j++)
            columns[j].push_back((double)fields[j]);

        nrow++;
    }

    fclose(fp);
    return true;
}

const array& Datafile::GetColumn(int j) {
    assert(1 <= j && j <= ncol);
    return columns[j-1];
}

std::vector<real> Datafile::GetRow(int i) const {
    assert(ncol > 0 && 1 <= i && i <= nrow);
    std::vector<real> r(nrow);
    for(int j = 0; j < ncol; j++)
        r[j] = columns[j][i-1];
    return r;
}
