#ifndef SPLINE_H
#define SPLINE_H

#include "Common.h"
#include "array.h"

class SplineImpl;

enum {
    Linear,
    Cubic
};

/***************************************************************
 * Spline
 *
 * Interpolate a data set.
 ***************************************************************/
class Spline {
public:
    Spline();
    Spline(const array& X, const array& Y, int method = Linear);
    Spline(int n, const real* X, const real* Y, int method = Linear);
    Spline(const Spline& F);
    ~Spline();

    Spline& operator=(const Spline& F);

    real Evaluate(real x) const;
    real EvaluateDerivative(real x) const;

    real operator()(real x) const { return Evaluate(x); }
    int GetMethod() const { return method; }
    void GetData(array& xx, array& yy) const { xx = X; yy = Y; }

    /* Find a local maximum or minimum of the interpolated function */
//    real FindMaximum(real xguess, real* ymax = 0);
//    real FindMinimum(real xguess, real& ymin = 0);

protected:
    /* Number of intervals (= number of points - 1) */
    int N;

    /* (Copy of) original data set */
    array X, Y;

    /* Interpolation method */
    int method;

    /* Internal implementation of spline */
    SplineImpl* impl;

    /* Determine which interval the given point lies in */
    int LookupIndex(real x) const;
};

#endif // SPLINE_H
