#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <cassert>
#include <cstdio>
#include <cmath>
#include <cstdlib>
#include <cstring>

#include "array.h"


array::array() {
}

array::array(size_type n) : vector<real>(n) {
}

array::array(size_type n, real r) : vector<real>(n, r) {
}

array::array(size_type n, const real* v) : vector<real>(v, v+n) {
}

array::array(const vector<real>& v) : vector<real>(v) {
}

array& array::operator=(const vector<real>& v) {
    vector<real>::operator=(v);
    return *this;
}

array::~array() {
}

void array::linspace(real min, real max, int Nk) {
    assert(Nk > 1);
    resize(Nk);
    for(int i = 0; i < Nk; i++)
        (*this)[i] = min + i*(max - min)/(Nk-1);
}

void array::logspace(real min, real max, int Nk) {
    assert(Nk > 1);
    resize(Nk);
    for(int i = 0; i < Nk; i++)
        (*this)[i] = min*exp(i*log(max/min)/(Nk-1));
}

void array::setdata(size_type n, real r) {
    resize(n, r);
}

void array::setdata(size_type n, const real* v) {
    resize(n);
    for(iterator it = begin(); it != end(); it++)
        *it = *v++;
}

array2::array2() : array() {
    Nx = Ny = 0;
}

array2::array2(int Nx_, int Ny_) : array(Nx_*Ny_) {
    Nx = Nx_;
    Ny = Ny_;
}

array2::array2(int Nx_, int Ny_, real r) : array(Nx_*Ny_, r) {
    Nx = Nx_;
    Ny = Ny_;
}

array2::array2(const array2& a) : array(a) {
    Nx = a.Nx;
    Ny = a.Ny;
}

array2& array2::operator=(const array2& a) {
    array::operator=(a);
    Nx = a.Nx;
    Ny = a.Ny;
    return *this;
}

void array2::getsize(int& nx, int& ny) {
    nx = Nx;
    ny = Ny;
}

void array2::resize(int Nx_, int Ny_) {
    Nx = Nx_;
    Ny = Ny_;
    array::resize(Nx*Ny);
}

void array2::resize(int Nx_, int Ny_, real r) {
    Nx = Nx_;
    Ny = Ny_;
    array::resize(Nx*Ny, r);
}

array3::array3() : array() {
    Nx = Ny = Nz = 0;
}

array3::array3(int Nx_, int Ny_, int Nz_) : array(Nx_*Ny_*Nz_) {
    Nx = Nx_;
    Ny = Ny_;
    Nz = Nz_;
}

array3::array3(int Nx_, int Ny_, int Nz_, real r) : array(Nx_*Ny_*Nz_, r) {
    Nx = Nx_;
    Ny = Ny_;
    Nz = Nz_;
}

array3::array3(const array3& a) : array(a) {
    Nx = a.Nx;
    Ny = a.Ny;
    Nz = a.Nz;
}

array3& array3::operator=(const array3& a) {
    array::operator=(a);
    Nx = a.Nx;
    Ny = a.Ny;
    Nz = a.Nz;
    return *this;
}

void array3::resize(int Nx_, int Ny_, int Nz_) {
    Nx = Nx_;
    Ny = Ny_;
    Nz = Nz_;
    array::resize(Nx*Ny*Nz);
}

void array3::resize(int Nx_, int Ny_, int Nz_, real r) {
    Nx = Nx_;
    Ny = Ny_;
    Nz = Nz_;
    array::resize(Nx*Ny*Nz, r);
}
