#ifndef ARRAY_H
#define ARRAY_H

#include <vector>
using std::vector;

#include "Common.h"

/* Simple 1-dimensional array */
class array : public vector<real> {
public:
    array();
    explicit array(size_type n);
    array(size_type n, real r);
    array(size_type n, const real* v);
    array(const vector<real>& v);
    array& operator=(const vector<real>& v);
    ~array();

    void linspace(real min, real max, int Nk = 100);
    void logspace(real min, real max, int Nk = 100);

    void setdata(size_type n, real r);
    void setdata(size_type n, const real* v);

    real* data() { return &front(); }
    const real* data() const { return &front(); }
};


/* Simple 2-dimensional array */
class array2 : public array {
public:
    array2();
    array2(int Nx, int Ny);
    array2(int Nx, int Ny, real r);
    array2(const array2& a);
    array2& operator=(const array2& a);

    void getsize(int& nx, int& ny);
    void resize(int Nx, int Ny);
    void resize(int Nx, int Ny, real r);

    real operator()(int i, int j) const { return (*this)[i*Ny + j]; }
    real& operator()(int i, int j) { return (*this)[i*Ny + j]; }

protected:
    int Nx, Ny;
};


/* Simple 3-dimensional array */
class array3 : public array {
public:
    array3();
    array3(int Nx, int Ny, int Nz);
    array3(int Nx, int Ny, int Nz, real r);
    array3(const array3& a);
    array3& operator=(const array3& a);

    void resize(int Nx, int Ny, int Nz);
    void resize(int Nx, int Ny, int Nz, real r);

    real operator()(int i, int j, int k) const { return (*this)[i*Ny*Nz + j*Nz + k]; }
    real& operator()(int i, int j, int k) { return (*this)[i*Ny*Nz + j*Nz + k]; }

protected:
    int Nx, Ny, Nz;
};

#endif // ARRAY_H
